// analysis of head-to-head polling data and partisan lean

{ // setup
graph set window fontface "Georgia"

clear all

cd "D:\Dropbox\YLS RA\Partisan Lean H2H Polling"
global temp "Temp"
global input "Input"
global output "Output"
}


{ // clean and stack data

// load electoral college votes data
import excel using "$input/Electoral College Votes.xlsx", clear firstrow
save "$temp/electoral_votes", replace

// load partisan lean data (from 538)
import excel using "$input/State Partisan Lean Reference.xlsx", clear firstrow
keep state dem_lean_margin
save "$temp/partisan_lean", replace

// load 2016 polling data (sourced from: https://projects.fivethirtyeight.com/2016-election-forecast/)
import excel using "$input/Polling 2016.xlsx", clear firstrow
save "$temp/polling_2016", replace

// load 2016 presidential race results (from MIT Election Lab: https://dataverse.harvard.edu/dataset.xhtml?persistentId=doi:10.7910/DVN/42MVDX)
import delimited using "$input/1976-2016-president.csv", clear

// filter to 2016
keep if year == 2016

// correct for write-in of either major party
keep if inlist(party,"republican","democrat")
collapse (sum) candidatevotes (first) totalvotes, by(party state)

// determine dem margin
sort state party
by state: gen dem_2016_margin = (candidatevotes - candidatevotes[_n+1]) / totalvotes
replace dem_2016_margin = 100 * dem_2016_margin 

// save 2016 presidential race data
keep state dem_2016_margin
keep if !missing(dem_2016_margin)
save "$temp/2016 election", replace

// load polling data
import excel using "$input/Polling.xlsx", clear firstrow

// drop empty rows
drop if missing(state)

// calculate dem margin
gen dem_poll_margin = DemShare - TrumpShare

// calculate probability of dem wining (based on margin of error of poll)
gen std_error = 1.96 * MarginofError
gen z = (dem_poll_margin - 0) / std_error
gen prob_dem_win = normal(z)

// determine days from Jan. 1 2019 to median polling day for 
gen median_day = DateBegun + (DateEnded - DateBegun) / 2
format median_day %td
gen days_since = round(median_day - mdy(1,1,2019))

// collapse to average for each state-candidate
collapse (mean) dem_poll_margin prob_dem_win [fweight = days_since], by(state Dem)

// merge in polling data
merge m:1 state using "$temp/partisan_lean"
assert _m != 1
keep if _m == 3
drop _m

// merge in 2016 election data
merge m:1 state using "$temp/2016 election"
assert _m != 1
keep if _m == 3
drop _m

// merge in electoral college data
merge m:1 state using "$temp/electoral_votes"
assert _m != 1
keep if _m == 3
drop _m

// merge in 2016 polling data
merge m:1 state using "$temp/polling_2016"
assert _m != 1
keep if _m == 3
drop _m


// calculate dem margin vs lean
gen vs_lean = dem_poll_margin - dem_lean_margin

// calculate dem margin vs 2016 election
gen vs_2016_election = dem_poll_margin - dem_2016_margin

// calculate dem margin vs 2016 polls
gen vs_2016_polls = dem_poll_margin - dem_margin_2016_poll

// flag states that would have flipped since 2016
gen flip_state_blue = dem_poll_margin > 0 & dem_2016_margin < 0 if !missing(dem_poll_margin)
gen flip_state_red = dem_poll_margin < 0 & dem_2016_margin > 0 if !missing(dem_poll_margin)

// label variables
la var dem_poll_margin `""Candidate Margin in" "Head-to-Head Polls vs. Trump""'
la var dem_lean_margin "State Democratic Lean"
la var dem_2016_margin "Clinton's Margin in 2016"
la var dem_margin_2016_poll "Clinton's Margin in Polls in 2016"

// create 45 degree line for graphs
gen degree_45 = round(runiform(-30,30),30)

gen candidate_label = 1 if Dem == "Biden"
replace candidate_label = 2 if Dem == "Sanders"
replace candidate_label = 3 if Dem == "Warren"
replace candidate_label = 4 if Dem == "Buttigieg"
replace candidate_label = 5 if Dem == "Bloomberg"
replace candidate_label = 6 if Dem == "Klobuchar"
la define candidate_label 1 "Biden" 2 "Sanders" 3 "Warren" 4 "Buttigieg" 5 "Bloomberg" 6 "Klobuchar"
la val candidate_label candidate_label 

// save data
save "temp/stacked_data", replace

}


drop if inlist(Dem, "Klobuchar", "Bloomberg")
{ // graph performance against 2016 election
twoway (scatter dem_poll_margin dem_2016_margin, msize(vsmall) mcolor("0 114 178") by(candidate_label)) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Election Results")) ///
xtitle("Clinton 2016 Election Margin") ytitle("Candidate 2020 Polling Margin") ///
subtitle(, fcolor(none) lstyle(none))
graph export "$output/Polling vs. Hillary Election 2016.png", replace

// individual candidate graphs
twoway (scatter dem_poll_margin dem_2016_margin if Dem == "Biden", msize(small) mcolor("0 114 178")) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Election Results")) ///
xtitle("Clinton 2016 Election Margin") ytitle("Biden 2020 Polling Margin")
graph export "$output/Biden Polling vs. Hillary Election 2016.png", replace

// individual candidate graphs
twoway (scatter dem_poll_margin dem_2016_margin if Dem == "Sanders", msize(small) mcolor("0 114 178")) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Election Results")) ///
xtitle("Clinton 2016 Election Margin") ytitle("Sanders 2020 Polling Margin")
graph export "$output/Sanders Polling vs. Hillary Election 2016.png", replace


}


{ // graph performance against 2016 polls
twoway (scatter dem_poll_margin dem_margin_2016_poll, msize(small) mcolor("0 114 178") by(candidate_label)) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Polls")) ///
subtitle(, fcolor(none) lstyle(none)) ///
xtitle("Clinton 2016 Polling Margin") ytitle("2020 Polling Margin")
graph export "$output/Polling vs. Hillary Polls 2016.png", replace

twoway (scatter dem_poll_margin dem_margin_2016_poll if Dem == "Biden", msize(small) mcolor("0 114 178")) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Polls")) ///
xtitle("Clinton 2016 Polling Margin") ytitle("Biden 2020 Polling Margin")
graph export "$output/Biden Polling vs. Hillary Polls 2016.png", replace

twoway (scatter dem_poll_margin dem_margin_2016_poll if Dem == "Sanders", msize(small) mcolor("0 114 178")) ///
(line degree_45 degree_45, color(gs10)) ///
, ylabel(,nogrid) xline(0, lcolor(gs8)) yline(0, lcolor(gs8)) ///
legend(label(1 "State Head-to-Head" "Margin vs. Trump") label(2 "Same as Clinton Polls")) ///
xtitle("Clinton 2016 Polling Margin") ytitle("Sanders 2020 Polling Margin")
graph export "$output/Sanders Polling vs. Hillary Polls 2016.png", replace





}


{ // determine how much polls would need to be off for Trump to win (assuming that the amount polls are off is constant across states)
// assuming that the unpolled safe states (the 24 states plus DC with no polling) are set in stone -- 89 votes safe & unpolled for Republicans, 77 votes safe & unpolled for Democrats
// what do we assume about results for the other states for people w/out polling (e.g. the 17 states where Klobuchar is not included)?
use "$temp/stacked_data", clear

keep state Dem electoral_votes dem_poll_margin

gen first_loss = .

// currently this only works for Biden, Sanders, and Warren since they have polling in all 26 states with any polling
forvalues i = 0(.1)7 {
	local j = round(`i' * 10)
	local k = round(`j' - 1)
	
	gen dem_poll_margin_`j' = dem_poll_margin - `i'
	gen win_state_`j' = (dem_poll_margin_`j' > 0) if !missing(dem_poll_margin_`j')
	gen electoral_votes_`j' = win_state_`j' * electoral_votes
	bys Dem: egen total_electoral_votes_`j' = total(electoral_votes_`j')
	replace total_electoral_votes_`j' = total_electoral_votes_`j' + 77
	gen win_`j' = (total_electoral_votes_`j' >= 270) if !missing(total_electoral_votes_`j')
	
	if `j' != 0 {
		replace first_loss = `i' if win_`j' == 0 & win_`k' == 1
	}
	
	drop dem_poll_margin_`j' win_state_`j' electoral_votes_`j'
}
drop total_electoral_votes_* win_*

}


{ // how much Trump beat polls by in his unexpectedly won states
use "$temp/stacked_data", clear

keep if inlist(state, "North Carolina", "Florida", "Wisconsin", "Michigan", "Pennsylvania")
bys state: keep if _n == 1
keep state dem_2016_margin dem_margin_2016_polls 

gen trump_beat_polls_2016 = dem_margin_2016_polls - dem_2016_margin

// Trump beat polls by between 1.8% and 6.1% in these states (avg. 4.2%)
summ trump_beat_polls_2016

}


{ // create table for poll results
use "$temp/stacked_data", clear

// determine total number of states that each candidate has any polling in
sort Dem state dem_poll_margin 
bys Dem state: gen count = 1 if _n == 1 & !missing(dem_poll_margin)
bys Dem: egen counter = total(count)
drop count
// determine flipped electoral college votes
gen red_2016_election = dem_2016_margin < 0
gen blue_2016_election = dem_2016_margin > 0 if !missing(dem_2016_margin)
gen red_2016_polls = dem_margin_2016_polls < 0
gen blue_2016_polls = dem_margin_2016_polls > 0 if !missing(dem_margin_2016_polls)

// for election results
gen expect_flip_red = blue_2016_election * electoral_votes * (1 - prob_dem_win)
gen expect_flip_blue = red_2016_election * electoral_votes * prob_dem_win
gen expect_flip = expect_flip_blue - expect_flip_red
drop expect_flip_blue expect_flip_red
bys Dem: egen expect_flip_net_election = total(expect_flip)
drop expect_flip

bys Dem: keep if _n == 1
gsort -expect_flip_net_election
keep Dem expect_flip_net* counter

gen expect_dem_votes = 232 + expect_flip_net_election
gen expect_trump_votes = 306 - expect_flip_net_election

la var Dem "Candidate"
la var expect_flip_net_election "Expected Net Flipped Votes Compared to 2016 Election Outcomes"
la var counter "No. States with Polling Data"
la var expect_trump_votes "Expected Trump Votes"
la var expect_dem_votes "Expected Democrat Votes"
export excel using "$output/Net Expected Electoral Vote Change.xlsx", replace firstrow(varlabels)

}


{ // check electoral votes flipped without weighting -- results qualitatively the same

// load polling data
import excel using "$input/Polling.xlsx", clear firstrow

// drop empty rows
drop if missing(state)

// calculate dem margin
gen dem_poll_margin = DemShare - TrumpShare

// calculate probability of dem wining (based on margin of error of poll)
gen std_error = 1.96 * MarginofError
gen z = (dem_poll_margin - 0) / std_error
gen prob_dem_win = normal(z)

// collapse to average for each state-candidate
collapse (mean) dem_poll_margin prob_dem_win, by(state Dem)

// merge in polling data
merge m:1 state using "$temp/partisan_lean"
assert _m != 1
keep if _m == 3
drop _m

// merge in 2016 election data
merge m:1 state using "$temp/2016 election"
assert _m != 1
keep if _m == 3
drop _m

// merge in electoral college data
merge m:1 state using "$temp/electoral_votes"
assert _m != 1
keep if _m == 3
drop _m


// determine total number of states that each candidate has any polling in
sort Dem state dem_poll_margin 
bys Dem state: gen count = 1 if _n == 1 & !missing(dem_poll_margin)
bys Dem: egen counter = total(count)
drop count

// determine flipped electoral college votes
gen red_2016_election = dem_2016_margin < 0
gen blue_2016_election = dem_2016_margin > 0 if !missing(dem_2016_margin)

// for election results
gen expect_flip_red = blue_2016_election * electoral_votes * (1 - prob_dem_win)
gen expect_flip_blue = red_2016_election * electoral_votes * prob_dem_win
gen expect_flip = expect_flip_blue - expect_flip_red
drop expect_flip_blue expect_flip_red
bys Dem: egen expect_flip_net = total(expect_flip)
drop expect_flip

bys Dem: keep if _n == 1
gsort -expect_flip_net
keep Dem expect_flip_net counter

gen expect_dem_votes = 232 + expect_flip_net
gen expect_trump_votes = 306 - expect_flip_net

}





















